from rest_framework import serializers
from .models import *


class UserMobilePhoneSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserMobilePhone
        fields = ("number",)


class UserInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserInfo
        exclude = ("owner",)


class UserSerializer(serializers.ModelSerializer):
    info = UserInfoSerializer(required=False)
    phone = UserMobilePhoneSerializer(required=False)
    has_unionid = serializers.SerializerMethodField()

    def get_has_unionid(self, instance):
        return instance.unionid_or_none is not None

    class Meta:
        model = User
        depth = 1
        exclude = ("created_at", "last_logined_at", "identifier", "platform")


class UserWxaCodeSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserWxaCode
        fields = ("image",)



# Generated by Django 2.1.2 on 2018-11-02 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='status',
            field=models.PositiveSmallIntegerField(default=0),
        ),
    ]



# Generated by Django 2.1.2 on 2018-12-20 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0003_userwxacode'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userwxacode',
            name='data',
        ),
        migrations.AddField(
            model_name='userwxacode',
            name='image',
            field=models.FileField(default='', upload_to='wxacodes', verbose_name='WXA Code'),
            preserve_default=False,
        ),
    ]



import hashlib
import logging
import random

import arrow
import requests
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.views.decorators.http import require_http_methods
from requests.exceptions import RequestException
from rest_framework import status, views
from rest_framework.decorators import api_view
from rest_framework.exceptions import APIException
from rest_framework.response import Response

from .jwt_auth import *
from .models import *
from .serializers import *
from .signals import *
from .sms_utils import *
from .services import *

# logger
logger = logging.getLogger(__name__)

# 计算次数的过期时间，当天有效，所以最长为一天的秒数
DAY = 3600 * 24

# 每天最多请求多少次短消息
REQUEST_VEROODE_MAX_TIMES_PER_DAY = 10

# 验证码的位数
VERCODE_MAX_VALUE = (
    999_999 if not getattr(settings, "VERCODE_MAX_VALUE", None) else getattr(settings, "VERCODE_MAX_VALUE")
)
VERCODE_DIGIT = len(str(VERCODE_MAX_VALUE))

# 验证码的有效时间
VERCODE_EXPIRATION = (
    60 * 30 if not getattr(settings, "VERCODE_EXPIRATION", None) else getattr(settings, "VERCODE_EXPIRATION")
)


def send_user_with_token(user):
    """ 发送用户信息和token
    """
    # 生成JWT
    token, expire_date = obtain_json_web_token(user)
    payload = UserSerializer(user).data
    # append token and expire
    payload.update({"token": token, "expire_date": expire_date.timestamp})    
    return Response({"result": "success", "payload": payload})


# Create your views here
@require_http_methods(["GET"])
def validate_wx_token(request):
    """验证微信设置URL时发送的token
    """
    signature = request.GET.get("signature")
    timestamp = request.GET.get("timestamp")
    nonce = request.GET.get("nonce")
    echostr = request.GET.get("echostr")
    token = settings.WECHAT_TOKEN

    string_list = [token, timestamp, nonce]
    string_list.sort()
    sha1 = hashlib.sha1()
    for string in string_list:
        sha1.update(string.encode("utf-8"))
    hashcode = sha1.hexdigest()
    return HttpResponse(echostr if hashcode == signature else "")


@api_view(["POST"])
def obtain_token(request):
    """获取token

    参数
    method: 目前只支持"code"
    platform: 目前支持 "mina" 和 "web"

    # web平台参数
    identifier:  目前支持 "openid" 和 "unionid"

    return:
        如果成功则返回{"userid": 1, "token": "xxxx"}
        web平台如果identifier为"unionid“的情况下拿不到unionid则会返回错误
    """
    method = request.data.get("method")
    platform = request.data.get("platform")
    if method not in OBTAIN_TOKEN_METHODS or platform not in OBTAIN_TOKEN_METHODS[method]:
        return Response(
            {"result": "failure", "reason": f"Unknown method: {method} for platform {platform}"},
            status=status.HTTP_405_METHOD_NOT_ALLOWED,
        )
    else:
        return OBTAIN_TOKEN_METHODS[method][platform](request)


@api_view(["POST"])
@jwt_required
def request_vercode(request):
    """ 请求发送验证码

    错误码：
    -10000: 发送短消息错误
    -10001: 已经到达最大次数
    """

    user = request.authenticated_user
    phone_number = request.data.get("number")

    today = arrow.utcnow()

    times_key = "{}:vercode:{}-{}-{}:times".format(user.id, today.year, today.month, today.day)
    times = cache.get_or_set(times_key, 0, DAY)
    if times >= REQUEST_VEROODE_MAX_TIMES_PER_DAY:
        logger.info("Cant send sms to {}, Too many times: {}".format(phone_number, times))
        return Response({"result": "failure", "reason": "Max times reached", "error_code": "-10001"})

    def send_code(phone_number, code, code_key, times_key):
        try:
            send_sms(phone_number, code)
        except Exception as e:
            logger.exception(e)
            return Response({"result": "failure", "reason": "无法发送验证码，请稍后再试", "error_code": -10000})
        cache.incr(times_key)
        logger.info("Send code {} to {}, Current times: {}".format(code, phone_number, times + 1))
        return Response({"result": "success"})

    code_key = "{}:vercode:{}".format(user.id, phone_number)
    exists_code = cache.get(code_key)
    if exists_code:
        cache.expire(code_key, timeout=VERCODE_EXPIRATION)
        return send_code(phone_number, exists_code, code_key, times_key)
    else:
        code = str(random.randint(0, VERCODE_MAX_VALUE)).zfill(VERCODE_DIGIT)
        cache.set(code_key, code, timeout=VERCODE_EXPIRATION)
        return send_code(phone_number, code, code_key, times_key)


@api_view(["POST"])
@jwt_required
def bind_phone(request):
    """提交用户手机号

    错误码:
    -10000: 验证码错误
    -10001: 电话已经绑定给其他的用户
    """
    user = request.authenticated_user
    vercode = request.data.get("vercode")
    phone_number = request.data.get("number")
    code_key = "{}:vercode:{}".format(user.id, phone_number)
    exists_code = cache.get(code_key)
    if not exists_code or exists_code != vercode:
        logger.info("vercode {} not match exists coce {}".format(vercode, exists_code))
        return Response({"result": "failure", "reason": "Wrong vercode", "error_code": -10000})

    # 先检查是否已经绑定给其他的openid
    try:
        exists_user = User.objects.get(phone__number=phone_number)
        if exists_user != user:
            logger.info("Phone already bind to other user")
            return Response({"result": "failure", "reason": "Phone already bind to other user", "error_code": -10001})
        else:
            logger.info("Phone already bind to self")
            return Response({"result": "success", "warning": "Phone already bind to self"})
    except ObjectDoesNotExist:
        UserMobilePhone.objects.create(owner=user, number=phone_number)
        return Response({"result": "success"})


@api_view(["POST"])
@jwt_required
def bind_user_info(request):
    """绑定用户信息

    -10000: 用户session不存在
    """
    user = request.authenticated_user
    session_key = cache.get("{}:accessToken".format(user.id))
    if session_key is None:
        return Response({"result": "failure", "error_code": -10000, "reason": "session key missing"})
    user_info = decrypt_user_info(session_key, request.data.get("encryptedData"), request.data.get("iv"))

    if "unionId" in user_info:
        unionid, is_created = UserUnionId.objects.get_or_create(value=user_info["unionId"], defaults={"owner": user})
        if not is_created and unionid.owner != user:
            # 返回已经绑定的用户和新的Token
            UserOpenId.objects.filter(owner=user).update(owner=unionid.owner)
            user.status = User.STATUS_ABANDON
            user.save()
            logger.info("Current user: {} abandon because union id exists".format(user))
            user = unionid.owner

    user_info, _ = UserInfo.objects.update_or_create(
        owner=user,
        defaults={
            "nickname": user_info["nickName"],
            "avatar_url": user_info["avatarUrl"],
            "gender": user_info["gender"],
            "city": user_info["city"],
            "province": user_info["province"],
            "country": user_info["country"],
            "language": user_info["language"],
        },
    )

    if user == request.authenticated_user:
        return Response({"result": "success", "data": UserInfoSerializer(user_info).data})

    return send_user_with_token(user)


@api_view(["GET"])
@jwt_required
def get_user_wxacode(request):
    """返回用户的二维码
    """
    user = request.authenticated_user
    if hasattr(user, "wxa_code"):
        return Response({
            "result": "success", 
            "data": UserWxaCodeSerializer(user.wxa_code, context={"request": request}).data["image"]
        })
    wxacode = get_wxa_code_for_user(user)
    return Response({
        "result": "success", 
        "data": UserWxaCodeSerializer(wxacode, context={"request": request}).data["image"]
    })


def obtain_token_by_code_for_web(request):
    """通过code换取token

    参考：
    https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140842
    """
    try:
        openid, access_token, refresh_token, expire_in = exchange_access_token(request.data.get("code"))

        # 抓取unionid和用户信息
        data = fetch_user_info(access_token, openid)
        identifier = request.data.get("identifier")
        user = None
        if identifier == "unionid":
            if "unionid" not in data or not data["unionid"]:
                return Response({"result": "failure", "reason": "No unionid"})
            unionid = data["unionid"]
            user, is_created = User.objects.get_or_create(
                unionid__value=unionid, defaults={"platform": 0, "identifier": 1}
            )

            if is_created:
                logger.info("create new user by unionid: {}".format(unionid))
                UserUnionId.objects.create(owner=user, value=unionid)
                user.openids.create(value=openid)
            else:
                user.update_last_login_date()

        elif identifier == "openid":
            user, is_created = User.objects.get_or_create(
                openid__value=openid, defaults={"platform": 0, "identifier": 0}
            )

            if is_created:
                logger.info("create new user by openid: {}".format(openid))
                user.openids.create(value=openid)
            else:
                user.update_last_login_date()
        else:
            raise APIException(f"Unknown identifier: ${identifier}")

        # 保存session到redis server
        cache.set_many(
            {"{}:accessToken".format(user.id): access_token, "{}:refreshToken".format(user.id): refresh_token},
            expire_in,
        )

        # 更新用户信息
        UserInfo.objects.update_or_create(
            owner=user,
            defaults={
                "nickname": data["nickname"],
                "avatar_url": data["headimgurl"],
                "gender": data["sex"],
                "city": data["city"],
                "province": data["province"],
                "country": data["country"],
                "language": data["language"],
            },
        )
        user_logged_in.send(user.__class__, user=user)

        return send_user_with_token(user)
    except RequestException as e:
        logger.exception(e)
        return Response({"result": "failure", "reason": str(e)}, status=status.HTTP_400_BAD_REQUEST)


def obtain_token_by_code_for_mina(request):
    """通过code换取token

    参考：
    https://developers.weixin.qq.com/miniprogram/dev/api/open-api/login/code2Session.html
    """
    try:
        openid, session_key, unionid = exchange_session_key(request.data.get("code"))
        if unionid is not None:
            user, is_created = User.objects.get_or_create(
                unionid__value=unionid, defaults={"platform": 1, "identifier": 1}
            )

            if is_created:
                logger.info("create new user by unionid: {}".format(unionid))
                UserUnionId.objects.create(owner=user, value=unionid)
                # TODO: 这里是否已经会存在相同的openid了？
                user.openids.create(value=openid)
            else:
                # openid 可能不一样，需要保存一下
                user.openids.get_or_create(value=openid, defaults={"value": openid})
                # 更新最后登陆时间
                user.update_last_login_date()

        else:
            # unionid 为空， 需要创建一个临时用户
            user, is_created = User.objects.get_or_create(
                openid__value=openid, defaults={"platform": 1, "identifier": 1}
            )
            if is_created:
                logger.info("create new user by openid: {}".format(openid))
                user.openids.create(value=openid)
            else:
                # 更新最后登陆时间
                user.update_last_login_date()
        # 保存session key
        cache.set("{}:accessToken".format(user.id), session_key)

        user_logged_in.send(user.__class__, user=user)

        return send_user_with_token(user)
    except RequestException as e:
        logger.exception(e)
        return Response({"result": "failure", "reason": str(e)}, status=status.HTTP_400_BAD_REQUEST)


OBTAIN_TOKEN_METHODS = {"code": {"web": obtain_token_by_code_for_web, "mina": obtain_token_by_code_for_mina}}



# Generated by Django 2.1.2 on 2018-12-20 13:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0002_user_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserWxaCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('data', models.BinaryField(max_length=262144, verbose_name='Data')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='wxa_code', related_query_name='wxa_code', to='wechat_user.User', verbose_name='User')),
            ],
        ),
    ]



import json
import logging
import uuid

from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.profile import region_provider
from django.conf import settings
from rest_framework.exceptions import APIException

from aliyunsdkdysmsapi.request.v20170525 import SendSmsRequest

# 注意：不要更改
REGION = "cn-hangzhou"
PRODUCT_NAME = "Dysmsapi"
DOMAIN = "dysmsapi.aliyuncs.com"

acs_client = AcsClient(settings.DYSMS_ACCESS_KEY, settings.DYSMS_ACCESS_SECRET, REGION)
region_provider.add_endpoint(PRODUCT_NAME, REGION, DOMAIN)
# logger
logger = logging.getLogger(__name__)


def send_sms(phone_number, content):
    request = SendSmsRequest.SendSmsRequest()
    request.set_TemplateCode(settings.DYSMS_TEMPLATE_CODE)
    request.set_SignName(settings.DYSMS_SIGN_NAME)
    request.set_OutId(uuid.uuid1())
    request.set_PhoneNumbers(phone_number)
    request.set_TemplateParam('{"code":"%s"}' % content)
    response = acs_client.do_action_with_exception(request).decode("utf-8")
    logger.debug("do_action_with_exception: {}".format(response))
    response = json.loads(response)
    if "Code" in response and response["Code"] != "OK":
        logger.error("Send sms to {} fail, response: {}".format(phone_number, response))
        raise APIException("SMS ERROR: {}".format(response))
    logger.info("Send sms to {} success, response: {}".format(phone_number, response))



from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

# Create your models here.


class User(models.Model):
    """微信会员
    """

    PLATFORM_WEB = 0  # 从web登录
    PLATFORM_MINA = 1  # 从小程序登录
    PLATFORM_CHOICES = ((PLATFORM_WEB, _("Web")), (PLATFORM_MINA, _("Mina")))

    IDENTIFIER_OPENID = 0  # 通过OPENID识别用户
    IDENTIFIER_UNIONID = 1  # 通过UnionID识别用户
    IDENTIFIER_CHOICES = ((IDENTIFIER_OPENID, _("OpenID")), (IDENTIFIER_UNIONID, _("UnionID")))

    STATUS_NORMAL = 0  # 正常的账号
    STATUS_ABANDON = 1  # 废弃的账号（转移到unionId名下）

    platform = models.IntegerField(verbose_name=_("Platform"), choices=PLATFORM_CHOICES)
    identifier = models.IntegerField(verbose_name=_("Identifier"), choices=IDENTIFIER_CHOICES)
    last_logined_at = models.DateTimeField(null=True, blank=True, db_index=True)
    status = models.PositiveSmallIntegerField(default=STATUS_NORMAL)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    @property
    def unionid_or_none(self):
        try:
            return self.unionid
        except:
            return None

    @property
    def nickname_or_none(self):
        try:
            return self.info.nickname
        except:
            return None

    @property
    def openids_or_none(self):
        try:
            return self.openids.all()
        except:
            return None

    def update_last_login_date(self):
        self.last_logined_at = timezone.now()
        self.save(update_fields=["last_logined_at"])

    def __str__(self):
        return f"Id:{self.pk}"

    def __repr__(self):
        return f"Id:{self.pk} Nickname:{self.nickname_or_none} UnionId:{self.unionid_or_none} OpenIds:{list(self.openids_or_none)}"


class UserUnionId(models.Model):
    """ 会员的UnionID
    """

    owner = models.OneToOneField(
        "User",
        verbose_name=_("Union ID"),
        related_name="unionid",
        related_query_name="unionid",
        on_delete=models.CASCADE,
    )
    value = models.CharField(verbose_name=_("Open ID"), max_length=32, db_index=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    def __str__(self):
        return f"unionid: {self.value}"


class UserOpenId(models.Model):
    """会员的OpenID
    """

    owner = models.ForeignKey(
        "User", verbose_name=_("Open Id"), related_name="openids", related_query_name="openid", on_delete=models.CASCADE
    )
    value = models.CharField(verbose_name=_("Open ID"), max_length=32, db_index=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    def __str__(self):
        return self.value


class UserInfo(models.Model):

    # gender
    GENDER_UNKNOWN = 0
    GENDER_MALE = 1
    GENDER_FEMALE = 2

    # choices of gender
    GENDER_CHOICES = ((GENDER_UNKNOWN, _("Unknown")), (GENDER_MALE, _("Male")), (GENDER_FEMALE, _("Female")))

    owner = models.OneToOneField(
        "User", verbose_name=_("User"), related_name="info", related_query_name="info", on_delete=models.CASCADE
    )
    nickname = models.CharField(verbose_name=_("Nick name"), max_length=64, blank=True, null=True)
    avatar_url = models.URLField(verbose_name=_("Avatar URL"), max_length=255, blank=True, null=True)
    gender = models.SmallIntegerField(verbose_name=_("Gender"), choices=GENDER_CHOICES, default=GENDER_UNKNOWN)
    city = models.CharField(verbose_name=_("City"), max_length=32, blank=True, null=True)
    province = models.CharField(verbose_name=_("Province"), max_length=32, blank=True, null=True)
    country = models.CharField(verbose_name=_("Country"), max_length=32, blank=True, null=True)
    language = models.CharField(verbose_name=_("Language"), max_length=16, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    def __str__(self):
        return ("Owner: {} " "nickname: {} " "gender: {} " "country: {} " "province: {} " "country: {} ").format(
            self.owner, self.nickname, self.gender, self.country, self.province, self.city
        )


class UserMobilePhone(models.Model):
    owner = models.OneToOneField(
        "User", verbose_name=_("User"), related_name="phone", related_query_name="phone", on_delete=models.CASCADE
    )
    number = models.CharField(verbose_name=("Phone number"), max_length=16)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    def __str__(self):
        return "Owner: {} number: {}".format(self.owner, self.number)


class UserWxaCode(models.Model):
    """用户唯一的小程序码
    """
    owner = models.OneToOneField(
        "User", verbose_name=_("User"), related_name='wxa_code', related_query_name='wxa_code', on_delete=models.CASCADE
    )
    image = models.FileField(verbose_name=_("WXA Code"), upload_to="wxacodes")
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)




import base64
import json
import logging

import requests
from Crypto.Cipher import AES
from django.conf import settings
from django.core.cache import cache
from django.core.files.base import ContentFile
from rest_framework.exceptions import APIException

from .models import *

# logger
logger = logging.getLogger(__name__)


def exchange_access_token(code):
    url = f"https://api.weixin.qq.com/sns/oauth2/access_token?appid={settings.WECHAT_APPID}&secret={settings.WECHAT_APPSECRET}&code={code}&grant_type=authorization_code"

    # send request
    response = requests.get(url)
    response.encoding = "utf-8"
    data = response.json()
    # check error
    if "errcode" in data:
        error_msg = "{}({})".format(data["errmsg"], data["errcode"])
        logger.error(error_msg)
        raise APIException(error_msg)
    return data["openid"], data["access_token"], data["refresh_token"], data["expire_in"]


def fetch_user_info(access_token, openid):
    url = f"https://api.weixin.qq.com/sns/userinfo?access_token={access_token}&openid={openid}&lang=zh_CN"
    response = requests.get(url)
    response.encoding = "utf-8"
    data = response.json()
    # check error
    if "errcode" in data:
        error_msg = "{}({})".format(data["errmsg"], data["errcode"])
        logger.error(error_msg)
        raise APIException(error_msg)
    return data


def exchange_session_key(code):
    url = f"https://api.weixin.qq.com/sns/jscode2session?appid={settings.WECHAT_APPID}&secret={settings.WECHAT_APPSECRET}&js_code={code}&grant_type=authorization_code"

    response = requests.get(url)
    response.encoding = "utf-8"
    data = response.json()
    if "errcode" in data and data["errcode"] != 0:
        error_msg = "{}({})".format(data["errmsg"], data["errcode"])
        logger.error(error_msg)
        raise APIException(error_msg)
    return data["openid"], data["session_key"], data.get("unionid", None)


def unpad(s):
    return s[: -ord(s[len(s) - 1 :])]


def decrypt_user_info(session_key, encryptedData, iv):
    cipher = AES.new(base64.b64decode(session_key), AES.MODE_CBC, base64.b64decode(iv))
    decrypted = json.loads(unpad(cipher.decrypt(base64.b64decode(encryptedData))))
    if decrypted["watermark"]["appid"] != settings.WECHAT_APPID:
        raise Exception("Invalid Buffer")
    return decrypted


def get_server_access_token():
    access_token = cache.get("server:access_token")
    if access_token:
        return access_token

    url = f"https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={settings.WECHAT_APPID}&secret={settings.WECHAT_APPSECRET}"
    response = requests.get(url)
    response.encoding = "utf-8"
    data = response.json()
    if "errcode" in data and data["errcode"] != 0:
        error_msg = "{}({})".format(data["errmsg"], data["errcode"])
        logger.error(error_msg)
        raise APIException(error_msg)
    cache.set("server:access_token", data["access_token"], data["expires_in"] - 900)
    return data["access_token"]


def get_wxa_code_for_user(user):
    """
    """
    access_token = get_server_access_token()

    url = f"https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token={access_token}"

    data = {"scene": str(user.id)}
    response = requests.post(url, json=data)

    if response.headers["Content-Type"] != "image/jpeg":
        data = response.json()
        error_msg = "{}({})".format(data["errmsg"], data["errcode"])
        logger.error(error_msg)
        raise APIException(error_msg)

    wxacode, is_created = UserWxaCode.objects.get_or_create(owner=user)
    wxacode.image.save(f"wxacode_{user.id}.jpg", ContentFile(response.content))
    wxacode.save()
    return wxacode



from django.urls import path

from .views import *

urlpatterns = [
    path("validate", validate_wx_token),
    path("obtainToken", obtain_token),
    path("bindUserInfo", bind_user_info),
    path("me/phone", bind_phone),
    path("vercode", request_vercode),
    path("wxacode", get_user_wxacode)
]



# Generated by Django 2.1.2 on 2018-10-12 09:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('platform', models.IntegerField(choices=[(0, 'Web'), (1, 'Mina')], verbose_name='Platform')),
                ('identifier', models.IntegerField(choices=[(0, 'OpenID'), (1, 'UnionID')], verbose_name='Identifier')),
                ('last_logined_at', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
            ],
        ),
        migrations.CreateModel(
            name='UserInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nickname', models.CharField(blank=True, max_length=64, null=True, verbose_name='Nick name')),
                ('avatar_url', models.URLField(blank=True, max_length=255, null=True, verbose_name='Avatar URL')),
                ('gender', models.SmallIntegerField(choices=[(0, 'Unknown'), (1, 'Male'), (2, 'Female')], default=0, verbose_name='Gender')),
                ('city', models.CharField(blank=True, max_length=32, null=True, verbose_name='City')),
                ('province', models.CharField(blank=True, max_length=32, null=True, verbose_name='Province')),
                ('country', models.CharField(blank=True, max_length=32, null=True, verbose_name='Country')),
                ('language', models.CharField(blank=True, max_length=16, null=True, verbose_name='Language')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='info', related_query_name='info', to='wechat_user.User', verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='UserMobilePhone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('number', models.CharField(max_length=16, verbose_name='Phone number')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='phone', related_query_name='phone', to='wechat_user.User', verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='UserOpenId',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(db_index=True, max_length=32, verbose_name='Open ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='openids', related_query_name='openid', to='wechat_user.User', verbose_name='Open Id')),
            ],
        ),
        migrations.CreateModel(
            name='UserUnionId',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(db_index=True, max_length=32, verbose_name='Open ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='unionid', related_query_name='unionid', to='wechat_user.User', verbose_name='Union ID')),
            ],
        ),
    ]



import os 
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-wechat-user',
    version='0.79',
    packages=find_packages(),
    install_requires=requirements,
    include_package_data=True,
    license='BSD License',  # example license
    description='A wechat user model',
    long_description=README,
    url='',
    author='Ken Zhang',
    author_email='kennir@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',  # replace "X.Y" as appropriatetwine upload dist/* -r pypi
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


"""JSON WEB TOKEN authentication
"""

import logging
from functools import wraps

import arrow
import jwt
from django.conf import settings
from rest_framework import authentication, exceptions
from rest_framework.response import Response

from .models import User

logger = logging.getLogger(__name__)


# HTTP Authentication prefix
HTTP_JWT_PREFIX = "JWT"
# JWT secret
JWT_SECRET = settings.SECRET_KEY
# algorithm
JWT_ALGORITHM = "HS256"
# jwt expire
JWT_EXPIRE = 3600 * 24 * 30


def obtain_json_web_token(user):
    expire_date = arrow.utcnow().shift(seconds=JWT_EXPIRE)
    payload = {"id": user.id, "exp": expire_date.timestamp}
    return jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM), expire_date


def payload_to_user(payload):
    user_id = payload.get("id")
    return User.objects.filter(id=user_id).first()


def decode_to_user(request):
    """从头部获取HTTP_AUTHORIZATION的值，尝试解码为用户
    """
    auth_header = request.META.get("HTTP_AUTHORIZATION", "")
    if not auth_header:
        return None
    # 分割字符串 JWT xxxxxxxxxx
    try:
        prefix, token = auth_header.split()
    except ValueError:
        return None
    # 检查Prefix必须为HTTP_JWT_PREFIX(JWT)
    if prefix != HTTP_JWT_PREFIX:
        return None
    try:
        payload = jwt.decode(token, JWT_SECRET, algorithms=(JWT_ALGORITHM,))
    except jwt.exceptions.InvalidTokenError as error:
        logger.exception(error)
        return None
    # 尝试转换为用户
    return payload_to_user(payload)


def find_user(request):
    user = decode_to_user(request)
    if user:
        request.authenticated_user = user
    return user


def jwt_required(func):
    """验证登录用户的装饰器
    """

    @wraps(func)
    def verify_login(request, *args, **kwargs):
        user = decode_to_user(request)
        if not user:
            return Response({"result": "failure", "reason": "Not authenticated!"})
        else:
            request.authenticated_user = user
            return func(request, *args, **kwargs)

    return verify_login


class JsonWebTokenAuthentication(authentication.BaseAuthentication):
    """Json web token认证器
    """

    def authenticate(self, request):
        user = decode_to_user(request)
        if not user:
            raise exceptions.AuthenticationFailed("Not authenticated")
        request.authenticated_user = user
        return (user, None)
